/*-
 * Copyright (c) 2008 Jean-Yves Moulin <jym@ldh.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#ifndef __NetBSD__
#include <libutil.h>
#endif
#include "lbdr.h"

extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;
extern int optreset;
extern int errno;


struct total_service {
	long total_cnx;
	long total_pkts;
	long total_bytes;
} ;

/*const char * _malloc_options = "XAJVP";*/
int af = PF_UNSPEC;
int verbose = 0;
int force = 0;
int hformat = 0;
int help = 0;
int repeat = 0;
int traffic = 0;
static struct sockaddr_storage service;
static struct total_service *total, *total_old;

static struct option longopts[] = {
	{ "add",              no_argument,            NULL,           'A' },
	{ "delete",           no_argument,            NULL,           'D' },
	{ "clone",            no_argument,            NULL,           'C' },
	{ "modify",           no_argument,            NULL,           'M' },
	{ "active",           no_argument,            NULL,           'a' },
	{ "inactive",         no_argument,            NULL,           'i' },
	{ "change-scheduler", no_argument,            NULL,           'S' },
	{ "change-timeout",   no_argument,            NULL,           'T' },
	{ "clear",            no_argument,            NULL,           'c' },
	{ "force",            no_argument,            &force,          1  },
	{ "help",             no_argument,            &help,           1  },
	{ "human",            no_argument,            NULL,           'h' },
	{ "persistence",      no_argument,            NULL,           'p' },
	{ "verbose",          no_argument,            NULL,           'v' },
	{ "stats",            no_argument,            NULL,           's' },
	{ "traffic",          no_argument,            NULL,           't' },
	/*{ "device",           required_argument,      NULL,           'H' },*/
	{ "hashtable",        required_argument,      NULL,           'H' },
	{ NULL,               0,                      NULL,           0 }
};


#define IP_BUF 50
static char ip_buf[IP_BUF];
static char *paddr(struct sockaddr_storage *ip)
{
	struct sockaddr_in *in = (struct sockaddr_in *)ip;
	struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)ip;

	bzero(ip_buf, IP_BUF);

	if(ip->ss_family == AF_INET) {
		snprintf(ip_buf, IP_BUF, "%u.%u.%u.%u:%i",
			((unsigned char *)&in->sin_addr.s_addr)[0],
			((unsigned char *)&in->sin_addr.s_addr)[1],
			((unsigned char *)&in->sin_addr.s_addr)[2],
			((unsigned char *)&in->sin_addr.s_addr)[3],
			ntohs(in->sin_port));

	} else {
		snprintf(ip_buf, IP_BUF, "%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x.%i",
			((unsigned char *)&in6->sin6_addr.s6_addr)[0],
			((unsigned char *)&in6->sin6_addr.s6_addr)[1],
			((unsigned char *)&in6->sin6_addr.s6_addr)[2],
			((unsigned char *)&in6->sin6_addr.s6_addr)[3],
			((unsigned char *)&in6->sin6_addr.s6_addr)[4],
			((unsigned char *)&in6->sin6_addr.s6_addr)[5],
			((unsigned char *)&in6->sin6_addr.s6_addr)[6],
			((unsigned char *)&in6->sin6_addr.s6_addr)[7],
			((unsigned char *)&in6->sin6_addr.s6_addr)[8],
			((unsigned char *)&in6->sin6_addr.s6_addr)[9],
			((unsigned char *)&in6->sin6_addr.s6_addr)[10],
			((unsigned char *)&in6->sin6_addr.s6_addr)[11],
			((unsigned char *)&in6->sin6_addr.s6_addr)[12],
			((unsigned char *)&in6->sin6_addr.s6_addr)[13],
			((unsigned char *)&in6->sin6_addr.s6_addr)[14],
			((unsigned char *)&in6->sin6_addr.s6_addr)[15],
			ntohs(in6->sin6_port));
	}

	return ip_buf;
}

static int same_addr(struct sockaddr_storage *a, struct sockaddr_storage *b)
{
	struct sockaddr_in *in4a, *in4b;
	struct sockaddr_in6 *in6a, *in6b;

	if(a->ss_family != b->ss_family) return 0;

        switch (a->ss_family) {
        case AF_INET:
		in4a = (struct sockaddr_in *)a;
		in4b = (struct sockaddr_in *)b;
                if( in4a->sin_addr.s_addr == in4b->sin_addr.s_addr &&
                           in4a->sin_port == in4b->sin_port)
			 return 1;
                break;
        case AF_INET6:
        	in6a = (struct sockaddr_in6 *)a;
        	in6b = (struct sockaddr_in6 *)b;

                if( in6a->sin6_addr.s6_addr[0]  == in6b->sin6_addr.s6_addr[0] &&
                    in6a->sin6_addr.s6_addr[4]  == in6b->sin6_addr.s6_addr[4] &&
                    in6a->sin6_addr.s6_addr[8]  == in6b->sin6_addr.s6_addr[8] &&
                    in6a->sin6_addr.s6_addr[12] == in6b->sin6_addr.s6_addr[12] &&
		in6a->sin6_port == in6b->sin6_port) 
			return 1;
                break;
        }

	return 0;
}

char *sched_name(int type)
{
	switch(type) {
		case LBDR_SCHED_LC:
			return " LC";
		case LBDR_SCHED_WRR:
			return "WRR";
		case LBDR_SCHED_WLC:
			return "WLC";
		case LBDR_SCHED_RR:
			return " RR";
		default:
			break;
	}

	return "***";
}

int is_digit(char *s)
{
	int i;
	for(i = 0; s[i]; i++) {
		if(s[i] != '0' &&
		   s[i] != '1' &&
		   s[i] != '2' &&
		   s[i] != '3' &&
		   s[i] != '4' &&
		   s[i] != '5' &&
		   s[i] != '6' &&
		   s[i] != '7' &&
		   s[i] != '8' &&
		   s[i] != '9')
		   	return 0;
	}

	return 1;
}

struct sockaddr_storage * get_ip(char *av, char *defp, int family)
{
	struct addrinfo hints, *res0;
	struct sockaddr_storage *addr = NULL;
	int error;
	char *port;

	port = strchr(av, ',');
	if(port) {
		*port++ = '\0';
		if(!is_digit(port)) {
			printf("Port (%s) must be a number\n", port);
			port = defp;
		}
	} else {
		port = defp;
	}
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = family;
	hints.ai_socktype = SOCK_STREAM;

	error = getaddrinfo(av, port, &hints, &res0);

	if (error) {
		printf("%s\n", gai_strerror(error));
		exit(EXIT_FAILURE);
	}

	if(res0) {
		addr = malloc(sizeof(struct sockaddr_storage));
		if(addr)
			memcpy(addr, res0->ai_addr, sizeof(struct sockaddr_storage));
	} else {
		printf("%s\n", gai_strerror(error));
	}

	freeaddrinfo(res0);

	return addr;
}



int get_sched(char *av)
{
	if(!strncasecmp("wlc", av, 3)) {
		return LBDR_SCHED_WLC;
	} else if(!strncasecmp("wrr", av, 3)) {
		return LBDR_SCHED_WRR;
	} else if(!strncasecmp("lc", av, 2)) {
		return LBDR_SCHED_LC;
	} else if(!strncasecmp("rr", av, 2)) {
		return LBDR_SCHED_RR;
	}
	return -1;
}


#define HU_DECIMAL 0x01
#define HU_NOSPACE 0x02

char *human(double nb, char *suffix, int flags)
{
	static char buffer[IP_BUF];
	int div = 0;
	const char *prefixes = "\0\0K\0M\0G\0T\0P\0E";

	bzero(buffer, IP_BUF);

	if(hformat) {
		while(nb >= 1000) {
			div++;
			nb = nb / 1024;
		}

		if((flags & HU_DECIMAL) && nb < 10)
			snprintf(buffer, IP_BUF, "%.1f%s%s%s", nb, flags & HU_NOSPACE ? "" : " ", &prefixes[div << 1], suffix);
		else
			snprintf(buffer, IP_BUF, "%.0f%s%s%s", nb, flags & HU_NOSPACE ? "" : " ", &prefixes[div << 1], suffix);
	} else {
		if((flags & HU_DECIMAL) && nb < 10)
			snprintf(buffer, IP_BUF, "%.1f%s", nb, suffix);
		else
			snprintf(buffer, IP_BUF, "%.0f%s", nb, suffix);
	}

	return buffer;
}

void print_data(struct lbdr_param *p, struct lbdr_param *o, struct total_service *total, struct total_service *total_old, int repeat)
{
	static int print_service = 0;
	int div = repeat;
	float coll_sec = 0.0;
	float perc_timeout = 0.0;

	if(af && p->ip_dst.ss_family != af) return;

	if(repeat == 0) {
		div = 1;
	}

	if(!same_addr(&service, &(p->ip_dst))) {
		memcpy(&service, &(p->ip_dst), p->ip_dst.ss_len);
		print_service = 1;

		if(verbose) puts("");

		printf("IP:     => %s", paddr(&service));

		if(!verbose) {
			printf("\t%9li Cnx", total->total_cnx - total_old->total_cnx);
			if(traffic) {
				printf("%12s ", human(total->total_pkts - total_old->total_pkts, "pkts", HU_DECIMAL));
				printf("%13s", human(total->total_bytes - total_old->total_bytes, "bytes", HU_DECIMAL));
			}
			if(p->coll - o->coll) printf(", %s Coll", human(p->coll - o->coll, "", HU_NOSPACE));
			if(p->err_nosyn - o->err_nosyn) printf(", %s Enosyn", human(p->err_nosyn - o->err_nosyn, "", HU_NOSPACE));
			if(p->err_nosrv - o->err_nosrv) printf(", %s Enosrv", human(p->err_nosrv - o->err_nosrv, "", HU_NOSPACE));
			puts("");
		}
	}

	if(verbose) {
		if(p->clone) {
			printf(", %s", paddr(&(p->ip_srv)));
		} else {
			if(p->tot_cnx > 0) {
				coll_sec = (float)(p->coll - o->coll) / (float)(total->total_cnx - total_old->total_cnx) / (float)div * 100.0;
				if(isnan(coll_sec)) coll_sec = 0.0;
				perc_timeout = (float)(p->no_fin - o->no_fin) * 100.0 / (float)(p->tot_cnx - o->tot_cnx);
				if(isnan(perc_timeout)) perc_timeout = 0.0;
			}
			if(print_service) {
				print_service = 0;
				printf("%s %s",  sched_name(p->sched), p->tls ? "SSL" : "");
				if(!repeat) printf("\n\t\tTsyn: %isec,  Test: %isec,  Tclo: %isec;", p->t_syn, p->t_est, p->t_clo);
				if(!repeat || (p->err_nosyn - o->err_nosyn)) printf("\n\t\tEnosyn: %s", human(p->err_nosyn - o->err_nosyn, "", HU_NOSPACE));
				if(!repeat || (p->err_nosrv - o->err_nosrv)) printf(",  Enosrv: %s", human(p->err_nosrv - o->err_nosrv, "", HU_NOSPACE));
				if(!repeat || (p->coll - o->coll)) printf(", Coll: %s (%.2f%%)", human(p->coll - o->coll, "", HU_NOSPACE), coll_sec);
				if(!repeat) printf(" HT: %s", human(p->hashtablesize, "", HU_NOSPACE));
				printf("\nSrv:");
			}
			printf("\t-%c %-21s %4u", p->status ? '>' : '|', paddr(&(p->ip_srv)), p->weight);
			if(traffic) {
				printf("   %12s  ", human(p->pkts - o->pkts, "pkts", HU_DECIMAL));
				printf("%13s  ", human(p->bytes - o->bytes, "bytes", HU_DECIMAL));
			} else {
				printf("   %8u   %8u   %8u (%.2f%%)   " , p->act_cnx, p->inact_cnx, p->no_fin, perc_timeout);
			}
			printf("%12s\n", human((p->tot_cnx - o->tot_cnx) / div, "", HU_NOSPACE));
		}
	}

}

int fill_total(struct lbdr_param *prm, int nb)
{
	struct sockaddr_storage svi;
	int s = 0, on;

	for(on = 0; on < nb; on++) {
		if(!same_addr(&svi, &(prm[on].ip_dst))) {
			memcpy(&svi, &(prm[on].ip_dst), sizeof(struct sockaddr_storage));
			s = on;
		}
		total[s].total_cnx += prm[on].tot_cnx;
		total[s].total_pkts += prm[on].pkts;
		total[s].total_bytes += prm[on].bytes;

	}
	bzero(&svi, sizeof(struct sockaddr_storage));
	for(on = 0; on < nb; on++) {
		if(!same_addr(&svi, &(prm[on].ip_dst))) {
			memcpy(&svi, &(prm[on].ip_dst), sizeof(struct sockaddr_storage));
			s = on;
		} else {
			total[on].total_cnx = total[s].total_cnx;
			total[on].total_pkts = total[s].total_pkts;
			total[on].total_bytes = total[s].total_bytes;
		}

	}

	return 0;
}

int main(int ac, char *av[])
{
	struct lbdr_param *prm, *oldprm;
	struct lbdr_stats *stats;
	struct lbdr_timeout *to;
	struct lbdr_info info;
	struct sockaddr_storage *i;
	void * comp = NULL;
	int fd, r, cmd = 0, on = 1, htsize = 15;
	char c;

	prm = malloc(sizeof(struct lbdr_param));
	stats = malloc(sizeof(struct lbdr_stats));
	to = malloc(sizeof(struct lbdr_timeout));

	if(!prm || !stats || !to) {
		printf("No memory");
		exit(EXIT_FAILURE);
	}

	bzero(prm, sizeof(struct lbdr_param));
	prm->status = 1; /* activated by default */
	prm->weight = 10; /* not null by default */

	info.nb_srv = 0;
	info.nb_cnx = 0;
	info.err_nomem = 0;
	info.err_badmbuf = 0;

	bzero(stats, sizeof(struct lbdr_stats));
	bzero(to, sizeof(struct lbdr_timeout));

	while((c = getopt_long(ac, av, "H:ADCTS46htpvamicsl", longopts, NULL)) != -1) {
		switch(c) {
		case 'A': /* Add */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRADDSRV;
			break;
		case 'D': /* Del */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRDELSRV;
			break;
		case 'C': /* clone */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRCLONE;
			break;
		case 'a': /* Active server */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			prm->status = 1;
			cmd = LBDRCHGSRV;
			break;
		case 'i': /* inactive */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			prm->status = 0;
			cmd = LBDRCHGSRV;
			break;
		case 'm': /* modify */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRCHGSRV;
			break;
		case 'p': /* persistence */
			prm->tls = 1;
		case 'S': /* sched */
			if(cmd && cmd != LBDRCHGSVICE) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRCHGSVICE;
			break;
		case 'T': /* timeout */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRCHGTO;
			break;
		case 'H':
			htsize = atoi(optarg);
			if(htsize > 25) {
				htsize = 25;
				puts("Maximum Hash table size: 2^25");
			} else if(htsize < 10) {
				htsize = 10;
				puts("Minimum Hash table size: 2^10");
			}
			break;
		case 'v':
			verbose = 1;
			break;
		/*
		case 'f': 
			force = 1;
			break;
			*/
		case 'c': /* clear all */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRCLEAR;
			break;
		case 't':
			traffic = 1;
		case 's': /* stats */
			if(cmd) { puts("Could not do two actions"); exit(EXIT_FAILURE); }
			cmd = LBDRSTATS;
			break;
		case '4':
			af = AF_INET;
			break;
		case '6':
			af = AF_INET6;
			break;
		case 'h':
			hformat = 1;
			break;
		default:
usage:
			free(prm);
			free(to);
			free(stats);
			printf("Usage: %s <option> <serviceip,port> [serverip] [scheduler:rr|lc|wrr|wlc] [weight]\n"
				"Option is one of these:\n"
				"\t--add or -A                Add server for service.\n"
				"\t--delete or -D             Delete server for service.\n"
				"\t--clone or -C              Clone service.\n"
				"\t--modify or -m             Change server weight.\n"
				"\t--active or -a             Activate server.\n"
				"\t--inactive or -i           Inactivate server (keep actives connections).\n"
				"\t--change-scheduler or -S   Change service scheduler.\n"
				"\t--change-timeout or -T     Change Timeout\n"
				"\t--hashsize or -H           Specify hash table size.\n"
				"\t--ipv4 or -4               Use IPv4\n"
				"\t--ipv6 or -6               Use IPv6\n"
				"\t--persistence or -p        Set SSL/TLS persistance\n"
				"\t--clear or -c              Clear all service and server.\n"
				"\t--force                    Force clear.\n"
				"\t--human                    Print number in human format.\n"
				"\t--stats or -s              Print statistic.\n"
				"\t--traffic or -t            Print traffic statistic.\n"
				"\t--verbose or -v            Be verbose.\n"
				"\nFor -ADCmai you need to specify <serviceip,port> and <serverip>\n"
				"For -ST you need to specify only <serviceip,port>\n"
				, av[0]);
			return EXIT_FAILURE;
			break;
		}
	}

	ac -= optind;
	av += optind;

	if(verbose) {
		if(af == AF_INET) puts("Using IPv4 address (AF_INET)");
		else if(af == AF_INET6) puts("Using IPv6 address (AF_INET6)");
		else puts("Using unspecified address (PF_UNSPEC)");
	}


	switch(cmd) {
		case LBDRADDSRV:
		case LBDRCLONE:
		case LBDRDELSRV:
		case LBDRCHGSRV:
			if(ac > 0) {
				i = get_ip(av[0], "80", af);
				if(!i) {
					puts("error in addr resolution");
					exit(EXIT_FAILURE);
				}
				memcpy(&(prm->ip_dst), i, i->ss_len);
				free(i);
			} else {
				puts("Must specify <ipaddr:port> for service");
				exit(EXIT_FAILURE);
			}

			if(ac > 1) {
				/* mettre le port du service par defaut: */
				i = get_ip(av[1], "80", af);
				if(!i) {
					puts("error in addr resolution");
					exit(EXIT_FAILURE);
				}
				memcpy(&(prm->ip_srv), i, i->ss_len);
				free(i);
			} else {
				puts("Must specify <ipaddr:port> for server");
				exit(EXIT_FAILURE);
			}

			if(ac > 2) { /* we have sched */
				prm->sched = get_sched(av[2]);
				if(prm->sched == -1) {
					if(cmd == LBDRCHGSRV && is_digit(av[2])) {
						prm->weight = atoi(av[2]);
					} else {
						puts("Invalid Scheduler. Valid value are: rr, lc, wrr, wlc");
						prm->sched = LBDR_SCHED_RR;
					}
				}
			}

			if(ac > 3) { /* we have weight */
				if(!is_digit(av[3])) {
					puts("Invalid weight. Must be a number");
					prm->weight = 10;
				} else prm->weight = atoi(av[3]);
			}
			prm->hashtablesize = htsize;
			if(verbose) {
				printf("PRM: %s ", paddr(&(prm->ip_dst)));
				printf("%s\n", paddr(&(prm->ip_srv)));
			}

			fd = open(LBDR_DEVICE, O_RDWR);
			if(fd == -1) {
				perror("open");
				if(!verbose) return -1;
			}

			r = ioctl(fd, cmd, prm);
			if(r == -1) {
				perror("lbdr ioctl");
			}

			break;

		case LBDRCHGSVICE:
			if(ac > 0) {
				i = get_ip(av[0], "80", af);
				memcpy(&(prm->ip_dst), i, i->ss_len);
				free(i);
			} else {
				puts("Must specify <ipaddr:port> for service");
				exit(EXIT_FAILURE);
			}

			if(ac > 1) { /* we have sched */
				prm->sched = get_sched(av[1]);
				if(prm->sched == -1) {
					puts("Invalid Scheduler. Valid value are: rr, lc, wrr, wlc");
					prm->sched = LBDR_SCHED_RR;
				}
			}

			fd = open(LBDR_DEVICE, O_RDWR);
			if(fd == -1) {
				perror("open");
				if(!verbose) return -1;
			}
			r = ioctl(fd, cmd, prm);
			if(r == -1) {
				perror("lbdr_sched");
			}

			break;
		case LBDRCHGTO:
			if(ac > 0) {
				i = get_ip(av[0], "80", af);
				memcpy(&(to->ip_dst), i, i->ss_len);
				free(i);
			} else {
				puts("Must specify <ipaddr:port> for service");
				exit(EXIT_FAILURE);
			}
			if(ac > 2) {
				if(!is_digit(av[1]) || !is_digit(av[2])) {
					puts("Timeout must be numbers.");
					return -1;
				}
				to->nt_syn = atoi(av[1]);
				to->nt_est = atoi(av[2]);
			}
			if(ac > 3) {
				to->nt_clo = atoi(av[3]);
			} else to->nt_clo = 0; /* no change */

			fd = open(LBDR_DEVICE, O_RDWR);
			if(fd == -1) {
				perror("open");
				if(!verbose) return -1;
			}
			r = ioctl(fd, cmd, to);
			if(r == -1) {
				perror("lbdr_set_timeout");
			}
			break;
		case LBDRCLEAR:
			/* ask for confirmation */
			if(!force) {
				printf("Are you sure do you want to clear all rules ? y/n : ");
				r = getchar();
				if(!(r == 'y' || r == 'Y')) return -1;
			}

			fd = open(LBDR_DEVICE, O_RDWR);
			if(fd == -1) {
				perror("open");
				if(!verbose) return -1;
			}
			r = ioctl(fd, cmd, &on);
			if(r == -1) {
				perror("lbdr_clear");
			}
			break;
		case LBDRSTATS:
			fd = open(LBDR_DEVICE, O_RDWR);
			if(fd == -1) {
				perror("open");
				if(!verbose) return -1;
			}
			r = ioctl(fd, LBDRINFO, &info);
			if(r == -1) {
				perror("lbdr");
			}
			if(verbose) printf("Nb of server: %i\n", info.nb_srv);

			printf("Total number of connections:\t%s\n"
				"Error for No enough memory:\t%u\n"
				"Error for Bad mbuf lenght:\t%u\n",
				human(info.nb_cnx, "", 0), info.err_nomem, info.err_badmbuf);

			stats->size = info.nb_srv * sizeof(struct lbdr_param);
			stats->data = malloc(stats->size);
			comp = malloc(stats->size);
			total = malloc(sizeof(struct total_service) * info.nb_srv);
			total_old = malloc(sizeof(struct total_service) * info.nb_srv);

			bzero(comp, stats->size);
			bzero(total, sizeof(struct total_service) * info.nb_srv);
			bzero(total_old, sizeof(struct total_service) * info.nb_srv);

			free(prm);
			prm = (struct lbdr_param *)stats->data;
			oldprm = (struct lbdr_param *)comp;

			if(verbose) {
				if(!traffic) puts("\tIPserveur:port         weight     active   inactive           timeout             Cnx");
				else         puts("\tIPserveur:port         weight          pkts          bytes            Cnx");
			}
			if(ac == 1) {
				if(!is_digit(av[0])) {
					puts("interval must be an integer");
					return -1;
				}
				repeat = atoi(av[0]);
				printf("repeat every %i secondes\n", repeat);


				for(;;) {
					bzero(prm, info.nb_srv * sizeof(struct lbdr_param));
					bzero(total, info.nb_srv * sizeof(struct total_service));
					r = ioctl(fd, cmd, stats);
					if(r == -1) {
						perror("lbdr");
						exit(EXIT_FAILURE);
					}

					fill_total(prm, info.nb_srv);
					for(on = 0; on < info.nb_srv; on++) {
						print_data(&(prm[on]), &(oldprm[on]), &(total[on]), &(total_old[on]), repeat);
					}
					memcpy(comp, prm, info.nb_srv * sizeof(struct lbdr_param));
					memcpy(total_old, total, sizeof(struct total_service) * info.nb_srv);
					bzero(&service, sizeof(struct sockaddr_storage));
					puts("");
					sleep(repeat);
				}

			} else {
				bzero(&service, sizeof(struct sockaddr_storage));
				r = ioctl(fd, cmd, stats);
				if(r == -1) { perror("lbdr"); }
				/* TODO: bug when no clone */
				fill_total(prm, info.nb_srv);
				for(on = 0; on < info.nb_srv; on++) {
					print_data(&(prm[on]), &(oldprm[on]), &(total[on]), &(total_old[on]), repeat);
				}
			}
			break;
		default:
			goto usage;
			break;
	}

	close(fd);
	free(prm);
	free(stats);
	free(to);
	puts("");

	return 0;
}

