/*-
 * Copyright (c) 2008 Jean-Yves Moulin <jym@ldh.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef __LB_DR_H__
#define __LB_DR_H__


/* le device a ouvrir pour configurer le LB: */
#define LBDR_DEVICE "/dev/lbdr"

/* les differents ordonnanceurs: */
#define LBDR_SCHED_RR	0x01
#define LBDR_SCHED_WRR	0x02
#define LBDR_SCHED_LC   0x04
#define LBDR_SCHED_WLC  0x08


/* la structure pour configurer le LB: */
struct lbdr_param {
	struct sockaddr_storage ip_dst;
	struct sockaddr_storage ip_srv;
	uint32_t hashtablesize;
	uint32_t coll;
	uint32_t tot_cnx;
	uint32_t inact_cnx;
	uint32_t act_cnx;
	uint32_t pkts;
	uint32_t bytes;
	uint32_t no_fin;
	uint32_t err_nosyn;
	uint32_t err_nosrv;
	uint32_t err_unreach;
	uint32_t t_syn;
	uint32_t t_est;
	uint32_t t_clo;
	uint32_t weight:15,
		 status:2,
		 sched:4,
		 tls:1,
		 clone:1;
};

struct lbdr_info {
	uint32_t nb_srv;
	uint32_t nb_cnx;
	uint32_t err_nomem;
	uint32_t err_badmbuf;
};

struct lbdr_timeout {
	struct sockaddr_storage ip_dst;
	uint32_t nt_syn;
	uint32_t nt_est;
	uint32_t nt_clo;
};

/* pour recuperer les statistiques: */
struct lbdr_stats {
	void *data;
	size_t size;
};

/* Les definitions des ioctl: */
#define		LBDRADDSRV	_IOW('L', 1, struct lbdr_param)
#define		LBDRDELSRV	_IOW('L', 2, struct lbdr_param)
#define		LBDRCHGSRV	_IOW('L', 3, struct lbdr_param)
#define		LBDRCHGSVICE	_IOW('L', 4, struct lbdr_param)
#define		LBDRCHGTO	_IOW('L', 5, struct lbdr_timeout)
#define		LBDRCLEAR	_IOW('L', 6, int)
#define		LBDRCLONE	_IOW('L', 10, struct lbdr_param)
#define		LBDRINFO	_IOR('L', 7, struct lbdr_info)
#define		LBDRSTATS	_IOWR('L', 8, struct lbdr_stats)


#endif
